{-------------------------------------------------------------------------------
   Title     :  TScnFile
   Version   :  3.52
   Type      :  Delphi Component  (Compatible with Delphi 2.0 3.0 4.0 5.0)
                                  (Version available for Delphi 1.0 )
   Cost      :  $35.00 US
   Date      :  January 14, 2000
   Author    :  G. Bradley MacDonald
                TimeAcct Information Systems
                3430 Copeland Avenue
                Vancouver, British Columbia
                Canada
                V5S 4B6
                Email :  Bradley.MacDonald@Telus.net
                         TopConsultant@Canada.com

   ============================================================================
   NOTE:        If you notice I have included the source to this component in
                the zip file.  This does not mean that the component is Free.  It
                simply means that I am trusting those who use the component in
                business applications to pay me the registration fee.  If you just
                want to play with the component or use it for non-business related
                applications - then you do not owe me any money.  If is it used in
                a business related application then you owe me the $35.00 US.
   ============================================================================

   Purpose   :  This component is a dual function component.  It can be used as a
                file find utility - and it can be used as a file scanning utility.
                You tell it the directory, file specifications and file attributes
                to include in the search, as well as whether to include sub
                directories or not.  Then you tell it if you also want to restrict
                the search by file size and file age.  Most important you can tell
                it to only include files that contain a particular string(s) (which you
                can make case sensitive or not).
                Then set the start search property to True and it will create a
                string list with the results of the search.
                So you can use this component to search a file(s) for a string, like
                a grep utility, or as a general seek and scan utility (File Find).
                There should be a small demo project included with this component
                with sample code on how to use it.
                See the SCANFILE function for the file scan engine
                See the LOCATEFILE procedure for the file find engine

                New :
                     Version 3.52 (January 2000)
                       Tested for Delphi 5.0
                       Touched up a few small routines.  Nothing Major.

                     Version 3.51 (April 1997)
                       Now has the ability to specify what directory types to search.  This
                         is useful if you want to search Hidden or System directories like
                         c:\win95\fonts.  This directory is a directory with the SYSTEM
                         attribute set - so it does not show up under a normal DIR statement
                         Now you can get fancy with your specifications settings.

                     Version 3.50 (April 1997)
                       I have now removed the 30,000 byte boundry error.  So there is no
                         longer a problem finding strings that cross the 30,000 byte
                         boundry
                       Big improvement in the 'Whole Word' search function.  It now works
                         on all files.  It allows you to specify allowable characters for
                         bounding the 'Word' being searched for.  See help for more details.

                     Version 3.40 (March 1997)
                       Now allows a crude 'Whole Word' search.  By this I mean that you
                         can specify how you want to search for the string.  It can be
                         whole work, prefix, suffix or normal.  See Help for more
                         information.  This function only works on plain text files at
                         this point

                     Version 3.30 (November 1996)
                       Now has the ability to search for files based on a date range rather
                         than just on file age - both now work.  If an invalid date is entered
                         then the component raises its own exception (EInvalidStartEndDate)
                         I did not go overboard with the error checking on this property.  I put
                         it in for convience.  See help for restrictions.
                       Now allows you to specify NOT in the string search facility.  So the
                         search is successful - if it does not find any of the strings.
                       Removed space at end of each field in the FScanResults String List

                     Version 3.20 (October 1996)
                       Now has events defined.  One for the start of the search, one
                         for the completion of the search, and one each for when directories
                         and files are found, and one for when the actual scanning of the
                         file is being performed.  You could use this to say 'delete all files
                         that are found automatically' - or whatever you want!
                       Ability to specify a match limit.  For example - stop after you
                         find 10 matches - even if you haven't finished the search
                       Ability to Not specify Directory name in ScanResult Property.
                       Ability to specify more than one File Spec.  You can now seperate
                         file specification by a ';'.  This allows you to search for
                         '*.TXT' and *.DOC' files (for example) at the same time!
                       Ability to search for multiple search strings at once, using AND/OR
                         logic in your search.  For example you could search for all files
                         that contain both the strings 'Device=' and 'Path' in them.

                     Version 3.10 (September 1996)
                       Now allows you to just pick the component name from a drop
                         downlist.  TaDa!  (Actually it wasn't that hard :))
                         Also added is a Search all available drives function.  This
                         allows the component to search all currently available drives.
                         So instead of forcing you to call each drive sperately - there
                         is now a function that allows you to say "Go find me that file
                         - where ever it is on the drives that are available to me - Now!"
                       Now allows the user to interrupt the search.  See the LOCATEFILE
                         section and the Application.ProcessMessages code.  It is all
                         explained there!
                       Now Includes help file - created by AutoHelp from DinoByte Software

                     Version 3.00 (September 1996)
                       Now allows you to update a label on your form with the current
                       search information (ie Currently searching file: XXX) I will
		       be updating this feature in the future to allow you to just
		       pick the component from a drop down list (I Hope!)

                     Version 2.10 (Last Freeware Version - June 1996)
                       Now includes ability to include file specs in your search
                       (Actually it always did - I just forgot to make the file
                        specs a public property :) Whoops!)
                       Add Ability to show extra info in Search Results.
                          ie File Size, Date, Attributes....
                       Add Ability to append a second scan to the first.
                       Fixed a bug with File Size Scanning.

                     Version 2.00 (May 1996)
                       Now works with Delphi 2.0

                     Version 1.10 (February 1996)
                       Now you can include filesize and file age in your search
                       criteria.

                NOTE:  It searches all types of files.  It is NOT just a text file
                       scanner.  It uses block IO to scan through all files.

                Limitations:
                1.  You should make the search string as small as possible, when
                    searching non-text files.  The problem is with formatted files.
                    For example - word processing documents.  You may have a document
                    that contains the string 'The boy went down the road'.  If you
                    search for the whole string - in a normal text file - you would
                    find it.  But if you are searching a word processing file (or
                    spreadsheet, etc) - then formatting (such as bolding, italic,etc)
                    will add extra characters to the file.  So when the search
                    engine tries to match the string - it won't find a match - because
                    there are extra characters in the string stored in the file.

*** NOTE ***

This registration will allow you full support via Email and Phone.  It will also in-title
you to one free upgrade to the next versioin.  After that there is an upgrade fee of 25% of
the original purchase price.  Upgrades will be delivered by Email Only!!
(Unless you are willing to cover cost of mailing :))

    If you make any improvements or would like to see an improvement to this component
please let me know!  I would like to hear any comments on this component (good & bad)
So let me know what you think!

Contact:   G. Bradley MacDonald
           TimeAcct Information Systems
           3430 Copeland Avenue
           Vancouver, BC
           V5S 4B6
           Canada
           Bradley_MacDonald@telus.net
           TopConsultant@Canada.com

{------------------------------------------------------------------------------}


{------------------------------------------------------------------------------}
{  Component Properties Explanation                                            }
{------------------------------------------------------------------------------}
{         See Help File                                                        }
{------------------------------------------------------------------------------}
unit UnitComp_TScnFile;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, stdctrls;

type
{------------------------------------------------------------------------}
{  Define the new exception to be used                                   }
{------------------------------------------------------------------------}
  EInvalidStartEndDate = class(Exception);

{------------------------------------------------------------------------}
{  Define the File type variable                                         }
{------------------------------------------------------------------------}
  TFileAttr         = (ftReadOnly, ftHidden, ftSystem, ftVolumeID, ftArchive, ftNormal);
  TFileType         = set of TFileAttr;
  TDirAttr          = (dtReadOnly, dtHidden, dtSystem, dtArchive);
  TDirType          = set of TDirAttr;
  TSearchType       = (st_And, st_Or, st_Not);
  TStringSearchType = (sst_Normal, sst_Word, sst_Prefix, sst_Suffix);
  TAllowableChars   = Set of Char;
{------------------------------------------------------------------------}
{  Define the File compare type variable                                 }
{     Set the Properties FileAgeComp and/or FileSizeComp to any below    }
{  Except or fctNone - in order to include that type of comparison in the}
{  file scan.  fctNone - does not perform that type of compare           }
{------------------------------------------------------------------------}
  TFileCompType = (fctNone,fctEqual,fctGreaterThan,fctLessThan,fctGreaterThanOrEqual,fctLessThanOrEqual);

{------------------------------------------------------------------------}
{  Define all the event types to be used by this object                  }
{------------------------------------------------------------------------}
  TFoundFileEvent        = Procedure(DirPath: String; FileInfo: TSearchRec) of Object;
  TFoundDirectoryEvent   = Procedure(DirPath: String; DirInfo: TSearchRec)  of Object;
  TStartSearchEvent      = Procedure of Object;
  TEndSearchEvent        = Procedure of Object;
  TBeforeBufferScanEvent = Procedure(FileName :String; SearchStrings :PChar; FileBuffer :PChar;
                                     ReadCount: Integer; CaseSense :Boolean) of Object;
  TAfterBufferScanEvent  = Procedure(FileName :String; SearchStrings :PChar; FileBuffer :PChar;
                                     ReadCount: Integer; CaseSense :Boolean; ScanSuccessful :Boolean) of Object;

{------------------------------------------------------------------------}
{  Define the TfileScan Class                                            }
{------------------------------------------------------------------------}
  TFileScan = class(TComponent)
  private
    { Private declarations }
    FDirectory             :String;
    FSearchAllDrives       :Boolean;
    FIncFileSpecs          :String;
    FSearchStrings         :TStringList;
    FSearchStringType      :TStringSearchType;
    FAllowablePrefix       :TAllowableChars;
    FAllowableSuffix       :TAllowableChars;

    FSearchType            :TSearchType;
    FIncFileType           :TFileType;
    FIncDirType            :TDirType;
    FIncSubDir             :Boolean;
    FCaseSense             :Boolean;
    FFileSize              :LongInt;
    FFileSizeComp          :TFileCompType;
    FFileAge               :LongInt;
    FFileAgeComp           :TFileCompType;
    FStartScan             :Boolean;

    FShowSize              :Boolean;
    FShowDate              :Boolean;
    FShowAttr              :Boolean;

    FAppendItems           :Boolean;
    FCurrInfoLabel         :TLabel;
    FInterrupted           :Boolean;
    FIncDirName            :Boolean;
    FMatchLimit            :Integer;
    FScanResult            :TStringList;

    FStartYear             :Integer;
    FStartMonth            :Integer;
    FStartDay              :Integer;
    FEndYear               :Integer;
    FEndMonth              :Integer;
    FEndDay                :Integer;

    FAllowNewSearch        :Boolean;

    { Event Variables }
    FOnFoundFile           :TFoundFileEvent;
    FOnFoundDirectory      :TFoundDirectoryEvent;
    FOnStartSearch         :TStartSearchEvent;
    FOnEndSearch           :TEndSearchEvent;
    FOnBeforeBufferScan    :TBeforeBufferScanEvent;
    FOnAfterBufferScan     :TAfterBufferScanEvent;

  protected
    { Protected declarations }
    function  ScanFile(FileName: String):Integer;
    Procedure LocateFiles(TreeBranch :String);
    Procedure SearchAllAvailDrives;
    Procedure SetDirectory(NewDirectory :String);
    Procedure SetCurrInfoLabel(NewLabel :TLabel);
    Procedure SetStartScan(NewValue :Boolean);
    Function  ScanBuffer(BufferPtr :PChar; SearchStrPtr :PChar; BufferLen :Word):Integer;
    Function  IsLeapYear(Year: Integer): Boolean;
    Procedure SetStartYear(NewYear :Integer);
    Procedure SetEndYear(NewYear :Integer);
    Procedure SetStartMonth(NewMonth :Integer);
    Procedure SetEndMonth(NewMonth :Integer);
    Procedure SetStartDay(NewDay :Integer);
    Procedure SetEndDay(NewDay :Integer);
    Function  CheckforWord(BufferPtr :PChar; BufferLen :Word; SearchStrPtr :PChar; StringPosPChar :PChar; SearchStringPChar :PChar) : Integer;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    Destructor Destroy; Override;
    property Interrupted      : Boolean           read FInterrupted      write FInterrupted     default False;
    property ScanResult       : TStringList       read FScanResult       write FScanResult;
    Property AllowablePrefix  : TAllowableChars   read FAllowablePrefix  write FAllowablePrefix;
    Property AllowableSuffix  : TAllowableChars   read FAllowableSuffix  write FAllowableSuffix;

  published
    { Published declarations }
    property DirType          : TDirType          read FIncDirType       write FIncDirType      default [];
    property FileType         : TFileType         read FIncFileType      write FIncFileType     default [ftNormal];
    property IncSubDir        : Boolean           read FIncSubDir        write FIncSubDir       default True;
    property Directory        : String            read FDirectory        write SetDirectory;
    property SearchAllDrives  : Boolean           read FSearchAllDrives  write FSearchAllDrives default False;
    property FileSpecs        : String            read FIncFileSpecs     write FIncFileSpecs;
    property SearchStrings    : TStringList       read FSearchStrings    write FSearchStrings;
    property SearchStringType : TStringSearchType read FSearchStringType write FSearchStringType;
    property SearchType       : TSearchType       read FSearchType       write FSearchType;
    property CaseSense        : Boolean           read FCaseSense        write FCaseSense       default False;
    property FileSize         : LongInt           read FFileSize         write FFileSize;
    Property FileSizeComp     : TFileCompType     read FFileSizeComp     write FFileSizecomp    default fctNone;
    property FileAge          : LongInt           read FFileAge          write FFileAge;
    Property FileAgeComp      : TFileCompType     read FFileAgeComp      write FFileAgecomp     default fctNone;
    Property CurrInfoLabel    : TLabel            read FCurrInfoLabel    write SetCurrInfoLabel;
    property StartScan        : Boolean           read FStartScan        write SetStartScan     default False;

    property ShowSize         : Boolean           read FShowSize         write FShowSize        default False;
    property ShowDate         : Boolean           read FShowDate         write FShowDate        default False;
    property ShowAttr         : Boolean           read FShowAttr         write FShowAttr        default False;
    property AppendItems      : Boolean           read FAppendItems      write FAppendItems     default False;
    property IncDirName       : Boolean           read FIncDirName       write FIncDirName      default True;
    property MatchLimit       : Integer           read FMatchLimit       write FMatchLimit;

    property StartYear        : Integer           read FStartYear        write SetStartYear;
    property StartMonth       : Integer           read FStartMonth       write SetStartMonth;
    property StartDay         : Integer           read FStartDay         write SetStartDay;
    property EndYear          : Integer           read FEndYear          write SetEndYear;
    property EndMonth         : Integer           read FEndMonth         write SetEndMonth;
    property EndDay           : Integer           read FEndDay           write SetEndDay;



    { Event Declarations }
    property OnFoundFile        : TFoundFileEvent        read FOnFoundFile        write FOnFoundFile;
    property OnFoundDirectory   : TFoundDirectoryEvent   read FOnFoundDirectory   write FOnFoundDirectory;
    property OnStartSearch      : TStartSearchEvent      read FOnStartSearch      write FOnStartSearch;
    property OnEndSearch        : TEndSearchEvent        read FOnEndSearch        write FOnEndSearch;
    property OnBeforeBufferScan : TBeforeBufferScanEvent read FOnBeforeBufferScan write FOnBeforeBufferScan;
    property OnAfterBufferScan  : TAfterBufferScanEvent  read FOnAfterBufferScan  write FOnAfterBufferScan;
  end;

procedure Register;

{------------------------------------------------------------------------}
{------------------------------------------------------------------------}
{               S T A R T   O F  I M P L E M E N T A T I O N             }
{------------------------------------------------------------------------}
{------------------------------------------------------------------------}
implementation


{ Global Variable Declaration }
{ I know many people frown on using Global Variables - but in this case }
{  they are useful.  These variables are used in the recursive routine  }
{  LOCATEFILES.  If they were not global - then each directory searched }
{  would chew up more memory then it has to.                            }
Var
  FileDateAge, TodayAge                                        :Double;
  Year, Month, Day, Hour, Minute, Second, MSec                 :Word;
  DateStr, TimeStr                                             :String;
  TempFloat                                                    :Single;
  TempString                                                   :String;
  XReadOnly, XHidden, XSystem ,XVolumeId, XDirectory, XArchive :String;
  FileDate, StartingDate, EndingDate                           :TDateTime;
  LengthofLastString                                           :LongInt;

{ -----------------------------------------------------------------------}
{  Register - Registers the component in the delphi component library    }
{     Change the 'Shareware' to put it in any panel you want!            }
{ -----------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('TimeAcct', [TFileScan]);
end;

{ -----------------------------------------------------------------------}
{  Constructor                                                           }
{     Called everytime the component is created                          }
{ -----------------------------------------------------------------------}
constructor TFileScan.Create(AOwner: TComponent);
Var
   i        :Integer;
begin
  inherited Create(AOwner);
  FDirectory        := 'C:\';
  FSearchStrings    := TStringList.Create;
  FSearchStrings.Clear;
  FSearchStringType := sst_Normal;
  FSearchType       := st_Or;
  FIncFileSpecs     := '*.*';
  FIncFileType      := [ftNormal];
  FIncDirType       := [];
  FIncSubDir        := True;
  FFileAge          := 0;
  FFileSize         := 0;
  FStartScan        := False;
  FShowSize         := False;
  FShowDate         := False;
  FShowAttr         := False;
  FAppendItems      := False;
  FInterrupted      := False;
  FSearchAllDrives  := False;
  FMatchLimit       := 0;
  FStartYear        := 0;
  FStartMonth       := 0;
  FStartDay         := 0;
  FEndYear          := 0;
  FEndMonth         := 0;
  FEndDay           := 0;
  FIncDirName       := True;
  FScanResult       := TStringList.Create;
  FScanResult.Clear;

  FAllowablePrefix  := [' '];
  FAllowableSuffix  := [' ',',','.'];

{  Now load all the TLabels on the form into the CurrInfoLabel Property }
  if (AOwner is TForm) then
  begin
     For I:=0 to TForm(AOwner).ComponentCount - 1 do
     begin
         if (TForm(AOwner).Components[I] is TLabel) then
            FCurrInfoLabel :=  TLabel(TForm(AOwner).Components[I]);
     end;
  end;

  FAllowNewSearch        := True;

{ Now display Registration Message for an unregister component - but only at run time!}
(*   If NOT(csDesigning in ComponentState) Then Begin
       ShowMessage('UnRegistered Delphi Component - TScnFile'
                   + #10#13 + #10#13
                   + 'To Remove this Nag Screen - Please Contact'
                   + #10#13
                   + 'Bradley MacDonald  to Register ($35.00 US Funds)'
                   + #10#13
                   + '(All Cheques Payable to Bradley MacDonald)'
                   + #10#13 + #10#13
                   + 'Via Email ...'
                   + #10#13
                   + '   Bradley_MacDonald@Telus.Net'
                   + #10#13
                   + '   TopConsultant@Canada.com'
                   + #10#13 + #10#13
                   + 'Via regular mail ...'
                   + #10#13
                   + '   3430 Copeland Avenue'
                   + #10#13
                   + '   Vancouver, BC'
                   + #10#13
                   +    'Canada   V5M 4B6'
                   );
   End;
*)
end;

{ -----------------------------------------------------------------------}
{  Destructor                                                            }
{     Called everytime the component is Destroyed                        }
{ -----------------------------------------------------------------------}
Destructor TFileScan.Destroy;
begin
  FScanResult.Free;  { Gotta Free the memory - or app will have memory leak!}
  FSearchStrings.Free;
  inherited Destroy;
End;


{ -----------------------------------------------------------------------}
{  SetDirectory                                                          }
{     This function is called to read the Directory private variable     }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetDirectory(NewDirectory :String);
begin
   FDirectory := NewDirectory;
end;

{ -----------------------------------------------------------------------}
{  IsLeapYear                                                            }
{     This function is called to determine if the year is a leap year or }
{  not                                                                   }
{ -----------------------------------------------------------------------}
function TFileScan.IsLeapYear(Year: Integer): Boolean;
begin
  Result := (Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0));
end;

{ -----------------------------------------------------------------------}
{  SetStartYear                                                          }
{     This function is called to write the FStartYear property.  Error   }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetStartYear(NewYear :Integer);
begin
   If NewYear <> FStartYear then Begin
     If (NewYear > 1900)  Or (NewYear = 0) then begin
        FStartYear := NewYear;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid Start Year: Year Must be > 1900 - Or Equal 0');
     End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SetStartMonth                                                         }
{     This function is called to write the FStartMonth property.  Error  }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetStartMonth(NewMonth :Integer);
begin
   If NewMonth <> FStartMonth then Begin
     If ((NewMonth > 0) and (NewMonth < 13))
     Or  (NewMonth = 0) then begin
        FStartMonth := NewMonth;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid Start Month: Month Must be 0 < Month > 13 - Or Equal to 0');
     End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SetStartDay                                                           }
{     This function is called to write the FStartDay property.  Error    }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetStartDay(NewDay :Integer);
const
  MonthDays: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
Var
  MaxDays  : Integer;

begin
   If NewDay <> FStartDay then Begin
     MaxDays := MonthDays[FStartMonth];
     if (FStartMonth = 2) and IsLeapYear(FStartYear) then Begin
        Inc(MaxDays); {In LeapYear - Feb has 29 days}
     End;

     If ((NewDay > 0) and (NewDay <= MaxDays))
     Or  (NewDay = 0) then begin
        FStartDay := NewDay;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid Start Day: Day Must be 0 > Day < (Max for Month)');
     End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SetEndDay                                                             }
{     This function is called to write the FEndDay property.  Error    }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetEndDay(NewDay :Integer);
const
  MonthDays: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
Var
  MaxDays  : Integer;

begin
   If NewDay <> FEndDay then Begin
     MaxDays := MonthDays[FEndMonth];
     if (FEndMonth = 2) and IsLeapYear(FEndYear) then Begin
        Inc(MaxDays); {In LeapYear - Feb has 29 days}
     End;

     If ((NewDay > 0) and (NewDay <= MaxDays))
     Or  (NewDay = 0) then begin
        FEndDay := NewDay;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid End Day: Day Must be 0 > Day < (Max for Month)');
     End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SetEndYear                                                            }
{     This function is called to write the FEndYear property.  Error     }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetEndYear(NewYear :Integer);
begin
   If NewYear <> FEndYear then Begin
     If (NewYear > 1900)  Or (NewYear = 0) then begin
        FEndYear := NewYear;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid End Year: Year Must be > 1900 or Equal 0');
     End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SetEndMonth                                                           }
{     This function is called to write the FEndMonth property.  Error  }
{  Checking must be performed before writing it                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetEndMonth(NewMonth :Integer);
begin
   If NewMonth <> FEndMonth then Begin
     If ((NewMonth > 0) and (NewMonth < 13))
     Or  (NewMonth = 0) then begin
        FEndMonth := NewMonth;
     End Else Begin
        Raise EInvalidStartEndDate.Create('Invalid End Month: Month Must be 0 < Month > 13 - Or Equal to 0');
     End;
   End;
end;



{ -----------------------------------------------------------------------}
{  SetStartScan                                                          }
{     This function is called to start the scanning process              }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetStartScan(NewValue :Boolean);
Var
   SaveAppendItems                        :Boolean;
begin
   If NewValue then Begin
      { Set FStartScan to False - So that it can be used again by changing it to True}
      FStartScan   := False;
      FInterrupted := False;
      { Clear the String List for the new search (If AppendItems is not True) }
      If Not FAppendItems Then Begin
        FScanResult.Clear;
      End;
      {Event Generation }
      If Assigned(FOnStartSearch) then FOnStartSearch;

      If SearchAllDrives then begin
        { If Search AllDrives is set to true - then scan all Avail Drives}
        SaveAppendItems := FAppendItems;
        FAppendItems    := True;
        SearchAllAvailDrives;
        FAppendItems    := SaveAppendItems;
      End Else Begin
        LocateFiles(FDirectory);
      End;
      If Assigned(FOnEndSearch) then FOnEndSearch;
   End;
end;

{ -----------------------------------------------------------------------}
{  SeachAllAvailDrives                                                   }
{     This procedure determines which drives the user currently has      }
{ access to and then scans them                                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SearchAllAvailDrives;
Var
   Drive             :Integer;
   DriveString       :String;
Begin
  Try

    For Drive := 0 to 25 Do Begin  {Scan all 26 drives - 'A' to 'Z' }
      DriveString := Chr(Drive + Ord('a')) + ':\';
{$IFDEF WIN32}
      If GetDriveType(PChar(DriveString)) <> 0 then Begin
        LocateFiles(DriveString);
      End; { If GetDriveType }
{$ELSE}
      If GetDriveType(Drive) <> 0 then Begin
        LocateFiles(DriveString);
      End; { If GetDriveType }
{$ENDIF}
    End;
  Finally
  End;
End;


{ -----------------------------------------------------------------------}
{  Locate Files                                                          }
{     This function looks for files that match the search criteria       }
{ -----------------------------------------------------------------------}
Procedure TFileScan.LocateFiles(TreeBranch :String);
var
  FSearchRec, DSearchRec                                       :TSearchRec;
  FindResult                                                   :Integer;
  FileAttr                                                     :Byte;
  DirAttr                                                      :Byte;
  iScanResult                                                  :Integer;
  AbortSearch                                                  :Boolean;
  FoundMatch                                                   :Boolean;
  //i                                                            :Integer;
  PosStart                                                     :Integer;
  TempFileSpecs                                                :String;
  FileSpecstoInclude                                           :String;

Begin
//  AbortSearch      := False;
//  FoundMatch       := False;

  If FAllowNewSearch Then Begin
    FAllowNewSearch := False;
    Try

      { Setup the File Age and Size Variables }
      TodayAge         := Date + 0.99999;

    { Make Sure that the Directory Branch ends with a '\' }
      If TreeBranch[Length(TreeBranch)] <> '\' Then Begin
         TreeBranch := TreeBranch + '\';
      End;

    { Find All File Types that have been selected }
      FileAttr := 0;
      If ftReadOnly in FIncFileType then
        FileAttr := FileAttr + faReadOnly;
      If ftHidden in FIncFileType then
        FileAttr := FileAttr + faHidden;
      If ftSystem in FIncFileType then
        FileAttr := FileAttr + faSysFile;
      If ftVolumeId in FIncFileType then
        FileAttr := FileAttr + faVolumeId;
      If ftArchive in FIncFileType then
        FileAttr := FileAttr + faArchive;

    {--------------------------------------------------------------------------}
    {  Now Start the Actual Search for files that match the criteria           }
    {--------------------------------------------------------------------------}
      Try

        FileSpecstoInclude := FIncFileSpecs;

        Repeat
          PosStart             := Pos(';',FileSpecsToInclude);
          { If PosStart > 0 - then there are multiple filespecs - must use each one }
          If PosStart = 0 then begin
            TempFileSpecs      := FileSpecsToInclude;
            FileSpecsToInclude := '';
          End Else Begin
            TempFileSpecs      := Copy(FileSpecsToInclude,1,PosStart - 1);
            FileSpecsToInclude := Copy(FileSpecsToInclude,PosStart + 1,10000);
          End;


          FindResult := FindFirst(TreeBranch + TempFileSpecs ,FileAttr, FSearchRec);

          While ((FindResult=0) and Not (FInterrupted)) Do Begin { Now loop through all the files that match }
            { Update the current Search Information for the CurrInfoComp }
            If CurrInfoLabel <> Nil then begin
               { Make sure to clear the label area - remember spaces take up very little space!!}
               CurrInfoLabel.Caption := TreeBranch + FSearchRec.Name +
                                        '                          ' +
                                        '                          ' +
                                        '                          ';
               CurrInfoLabel.Refresh;
            End;

            FoundMatch  := True;
            AbortSearch := False;
            {---------------------------------------------------------------------}
            { Now check to see if the search should contain a check for file size }
            {---------------------------------------------------------------------}
            If Not AbortSearch Then Begin
            If FileSizeComp <> fctNone then Begin
              Case FileSizeComp of
                fctEqual:
                  If Not (FSearchRec.Size = FileSize) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctGreaterThan:
                  If Not (FSearchRec.Size > FileSize) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctGreaterThanOrEqual:
                  If Not (FSearchRec.Size >= FileSize) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctLessThan:
                  If Not (FSearchRec.Size < FileSize) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctLessThanOrEqual:
                  If Not (FSearchRec.Size <= FileSize) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
              End;
            End;
            End;

            {---------------------------------------------------------------------}
            { Now check the File Age Limit                                        }
            {---------------------------------------------------------------------}
            If Not AbortSearch then Begin
            If (FileAgeComp <> fctNone) then begin
              FileDateAge := TodayAge - FileDatetoDateTime(FSearchRec.Time);
              Case FileAgeComp of
                fctEqual:
                  If Not (Trunc(FileDateAge) = FileAge) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctGreaterThan:
                  If Not (Trunc(FileDateAge) > FileAge) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctGreaterThanOrEqual:
                  If Not (Trunc(FileDateAge) >= FileAge) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctLessThan:
                  If Not (Trunc(FileDateAge) < FileAge) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
                fctLessThanOrEqual:
                  If Not (Trunc(FileDateAge) <= FileAge) then begin
                    FoundMatch  := False;
                    AbortSearch := True;
                  End;
              End;
            End;
            End;

            {---------------------------------------------------------------------}
            { Now check to see if the search should contain a check for file Dates}
            {---------------------------------------------------------------------}

            { First - if user has filled in End Date - but no start date - assume early start date }
            IF ((FEndYear <> 0) and (FStartYear = 0)) then begin
              FStartYear  := 1900;
              FStartMonth := 1;
              FStartDay   := 1;
            End;

            { If both are not equal 0 - then only include if file is between these dates }
            If ((FStartYear <> 0) and (FEndYear <> 0)) Then Begin
              Try
                { Build the Date/Time Variable to be used to check the file date }
                FileDate      := FileDatetoDateTime(FSearchRec.Time);
                StartingDate  := EncodeDate(FStartYear, FStartMonth, FStartDay);
                EndingDate    := EncodeDate(FEndYear, FEndMonth, FEndDay);
                If ((FileDate >= EndingDate) Or (FileDate <= StartingDate)) Then Begin
                  FoundMatch  := False;
                End;
              Except
                On EConvertError Do Begin   { I put this here so you could put any special code you want }
    //              AbortSearch :=  True;
                  Raise;                    {  and still raise the exception to your application !! }
                End;
              End;
            End;


            {---------------------------------------------------------------------}
            { Now check to see if the file contains the search string in it       }
            {---------------------------------------------------------------------}
            If Not AbortSearch Then Begin
              If (SearchStrings.Count > 0) Then Begin
                If NOT ((SearchStrings.Count = 1) and (SearchStrings[0] = '*')) Then Begin
                  iScanResult := ScanFile(TreeBranch + FSearchRec.Name);
                  IF iScanResult <= 0 then Begin
                    FoundMatch  := False;         { If zero or Lower - no match found          }
                    If iScanResult < 0 then begin  { If Less than Zero - then error during scan }
      //                AbortSearch := True;
                      FScanResult.Add('*** Error Scanning: ' + TreeBranch + FSearchRec.Name);
                    End;
                  End;
                End;
              End;
            End;

            {---------------------------------------------------------------------}
            { Now - If a Match has be Found - then add it to the output stringlist}
            {---------------------------------------------------------------------}
            If FoundMatch then begin

              {Event Generation }
              If Assigned(FOnFoundFile) then FOnFoundFile(TreeBranch, FSearchRec);

              If FIncDirName Then
                TempString := TreeBranch + FSearchRec.Name
              Else
                TempString := FSearchRec.Name;

             IF FShowSize Then Begin
                TempFloat  := FSearchRec.Size;
                TempString := TempString + '  ' + Format('%12.0n',[TempFloat]);
              End;

              IF FShowDate Then Begin
                DecodeDate(FileDatetoDateTime(FSearchRec.Time), Year, Month, Day);
                DecodeTime(FileDatetoDateTime(FSearchRec.Time), Hour, Minute, Second, MSec);
                DateStr    := Format('%4.2d',[Year]) + '/' + Format('%2.2d',[Month]) + '/' + Format('%2.2d',[Day]);
                TimeStr    := Format('%2.2d',[Hour]) + ':' + Format('%2.2d',[Minute]) + ':' + Format('%2.2d',[Second]);
                TempString := TempString + '  ' +  Format('%10s',[DateStr]) + '  ' + Format('%8s',[TimeStr]);
              End;
      {--------------------------------------------------}
              If FShowAttr Then Begin
                IF (faReadOnly AND FSearchRec.Attr) = faReadOnly then Begin
                  XReadOnly := 'R';
                End Else Begin
                  XReadOnly := '-';
                End;
                IF (faHidden AND FSearchRec.Attr) = faHidden then Begin
                  XHidden := 'H';
                End Else Begin
                  XHidden := '-';
                End;
                IF (faSysFile AND FSearchRec.Attr) = faSysFile then Begin
                  XSystem := 'S';
                End Else Begin
                  XSystem := '-';
                End;
                IF (faVolumeId AND FSearchRec.Attr) = FaVolumeId then Begin
                  XVolumeId := 'V';
                End Else Begin
                  XVolumeId := '-';
                End;
                IF (faDirectory AND FSearchRec.Attr) = faDirectory then Begin
                  XDirectory := 'D';
                End Else Begin
                  XDirectory := '-';
                End;
                IF (faArchive AND FSearchRec.Attr) = faArchive then Begin
                  XArchive := 'A';
                End Else Begin
                   XArchive := '-';
                End;
                TempString := TempString  +  '  ' +
                                          XReadOnly  +
                                          XHidden    +
                                          XSystem    +
                                          XVolumeId  +
                                          XDirectory +
                                          XArchive;
              End;
      {--------------------------------------------------}
              FScanResult.Add(TempString);
            End;

     { Now Check to see if the user wishes to interrupt the Search
       Remember - the interrupt handling code must be in your application - it is
                  Not in this component.  Your code - must set the Interrupted
                  Property for this to work!
       Example of Code  - Call from KeyPress for the Form ??
                 Procedure CheckforInterrupt(Key :Char);
                 Begin
                   If Key = Chr(VK_ESCAPE) then begin
                     TFileScan1.Interrupted := True;
                   End;
                 End;
     }
            Application.ProcessMessages;
            If FInterrupted Then Begin
    //          FoundMatch  := False;
    //          AbortSearch := True;
              FScanResult.Add('   ***** Search Aborted by User *****   ');
              Exit;
            End;

            { Stop Search if the Match Limit has been reached! }
            If ((FMatchLimit > 0) and (FMatchLimit <= FScanResult.Count)) Then Begin
              FScanResult.Add('  *** Search Match Limit has been meet!  Search Stopped.');
              Exit;
            End;

            FindResult  := FindNext(FSearchRec); {get the next file that matches}
          End;  {of While}
        Until PosStart = 0;  {of Repeat Until - for multiple file specs search}

        {---------------------------------------------------------------------}
        { Now - Search sub directories if that option has been selected       }
        {---------------------------------------------------------------------}
        If  FIncSubDir then Begin
           { Find All Directory Types that have been selected }
           DirAttr := faDirectory;
           If dtReadOnly in FIncDirType then
             DirAttr := DirAttr + faReadOnly;
           If dtHidden in FIncDirType then
             DirAttr := DirAttr + faHidden;
           If dtSystem in FIncDirType then
             DirAttr := DirAttr + faSysFile;
           If dtArchive in FIncDirType then
             DirAttr := DirAttr + faArchive;

           FindResult := FindFirst(TreeBranch + '*.*', DirAttr, DSearchRec);

           While ((FindResult=0) and Not (FInterrupted)) Do Begin
             If ((DSearchRec.Attr and faDirectory) = faDirectory)  and (pos('.', DSearchRec.Name) = 0) then begin
               {Event Generation }
               If Assigned(FOnFoundDirectory) then FOnFoundDirectory(TreeBranch,DSearchRec);

               LocateFiles(TreeBranch + DSearchRec.Name);
             End;
             FindResult := FindNext(DSearchRec);
           End;
        End;
      Finally
        SysUtils.FindClose(FSearchRec);
        SysUtils.FindClose(DSearchRec);
      End;
    Finally
      FAllowNewSearch := True;
    End;
  End;  { If Allow New Search  }
End;

{ ----------------------------------------------------------------------------}
{  CheckforWord                                                               }
{     This function is used to check to see if the found string also matches  }
{ the required 'Word', 'Prefix, and/or 'Suffix' string search types           }
{                                                                             }
{ Returns:  0 - Found String does not meet the searchstringtype parameters    }
{           1 - String matches fine - everything OK                           }
{ ----------------------------------------------------------------------------}
Function TFileScan.CheckforWord(BufferPtr :PChar; BufferLen :Word; SearchStrPtr :PChar; StringPosPChar :PChar; SearchStringPChar :PChar) : Integer;
Var
   PrefixOK            :Boolean;
   SuffixOK            :Boolean;
   PrefixChar          :Char;
   SuffixChar          :Char;
   FoundStringLength   :Integer;
Begin
   Result     := 0;
   PrefixOK   := False;
   SuffixOK   := False;
   PrefixChar := #0;
   SuffixChar := #0;

   FoundStringLength := StrLen(SearchStringPchar);
   { If String is the first character in the buffer - then no need to check characters
     before it - ie prefix is OK }
   IF StringPosPChar = BufferPtr then begin
     PrefixOK := True;
   End Else Begin
     PrefixChar := (StringPosPChar - 1)[0];
   End;

   { If String is the last characters in the buffer - then no char after string - so
     no need to check the suffix }
   If ((StringPosPChar + FoundStringLength) = (BufferPtr + BufferLen)) Then Begin
     SuffixOK := True;
   End Else Begin
     SuffixChar := (StringPosPChar + FoundStringLength)[0];
   End;

   { Prefix Search }
   IF (FSearchStringType = sst_Prefix) then begin
     If Not PrefixOK then begin
       If PrefixChar in FAllowablePrefix then begin
         Result := 1;
       End Else Begin
         Result := 0;
       End;
     End Else Begin
       Result := 1;
     End;
   End;

   { Suffix Search }
   IF (FSearchStringType = sst_Suffix) then begin
     If Not SuffixOK then begin
       If SuffixChar in FAllowableSuffix then begin
         Result := 1;
       End Else Begin
         Result := 0;
       End;
     End Else Begin
       Result := 1;
     End;
   End;

   { Word Search }
   IF (FSearchStringType = sst_Word) then begin
     If Not PrefixOK then begin
       If PrefixChar in FAllowablePrefix then begin
         PrefixOK := True;
       End;
     End;

     If PrefixOK then begin
       If Not SuffixOK then begin
         If SuffixChar in FAllowableSuffix then begin
           Result := 1;
         End Else Begin
           Result := 0;
         End;
       End Else Begin
         Result := 1;
       End;
     End Else Begin
       Result := 0;
     End;
   End;


End;

{ ----------------------------------------------------------------------------}
{  ScanBuffer                                                                 }
{     This function is used to scan the file buffer.                          }
{ Note: Return Values:  0 - End of File                                       }
{                       1 - String Found                                      }
{                      -1 - Error of Some Sort                                }
{                                                                             }
{ ----------------------------------------------------------------------------}
Function TFileScan.ScanBuffer(BufferPtr :PChar; SearchStrPtr :PChar; BufferLen :Word):Integer;
Var
   i                          :Word;
   BuffLen2                   :Word;
   StrLength                  :Word;
   StringPosPchar             :PChar;
   BuffPtr2                   :PChar;
   FailonStringTypeMatch      :Boolean;

Begin
   StringPosPchar     := Nil;
   LengthofLastString := 0;
   Result             := 0;
   Try
      BuffPtr2 := BufferPtr;
      BuffLen2 := BufferLen;
      i        := 0;

     { Scan until i is greater than the buffer length }
      While I < BuffLen2 Do Begin
        FailonStringTypeMatch := False;
        Try
          If StrLen(BuffPtr2) > 0 Then Begin
            If FCaseSense Then Begin
              StringPosPChar   := StrPos(BuffPtr2,SearchStrPtr);
            End Else Begin
              StringPosPChar   := StrPos(StrUpper(BuffPtr2),StrUpper(SearchStrPtr));
            End;  { FCaseSense }
            If StringPosPChar <> nil then begin
              If FSearchStringType <> sst_Normal then Begin
                Result := CheckforWord(BufferPtr, BufferLen, SearchStrPtr, StringPosPChar, SearchStrPtr);
                { Result         := 1;}
                If Result = 1 then Begin
                  LengthofLastString := StrLen(BuffPtr2);
//                  StringPosPChar     := Nil;
//                  BuffPtr2           := Nil;
                  Break;  { Exit the While loop - since you found the string }
                End Else Begin
                { If result is 0 - then it means that the string did not match the searchstringtype
                  So now you have to search the remaining part of the string !!
                }
                  Result                := 0;
                  { Now jump to the next null terminated string in the buffer }
                  FailonStringTypeMatch := True;
                End;  { If Result = 1 }
              End Else Begin
                Result             := 1;
                LengthofLastString := StrLen(BuffPtr2);
//                StringPosPChar     := Nil;
//                BuffPtr2           := Nil;
                Break;  { Exit the While loop - since you found the string }
              End;  { If SearchStringType = sst_Normal }
            end else begin
              Result         := 0;
            End;  { If StringPosPChar <> nil }
          End;  { If StrLen(BuffPtr2) > 0 }

          { Now jump to the next null terminated string in the buffer }
          IF FailonStringTypeMatch then begin
          { If FailonString is true - then you have to restart the search at the end of the
            found string - otherwise you might miss any further occurances of that string }
            StrLength          := StrLen(SearchStrPtr);
            i                  := i + (StringPosPchar - BuffPtr2 + StrLength);
            LengthofLastString := StrLen(BuffPtr2);
            StrLength          := StrLen(SearchStrPtr);
            BuffPtr2           := BuffPtr2 + (StringPosPchar - BuffPtr2 + StrLength);
          End Else Begin
            i                  := i + StrLen(BuffPtr2) + 1;
            LengthofLastString := StrLen(BuffPtr2);
            BuffPtr2           := BuffPtr2 + LengthofLastString + 1;
          End;  { If FailonStingTypeMatch }
          { If I is >= BuffLen2 - then you have traversed the whole buffer!}
          If i >= BuffLen2 then Begin
            Break;
          End;  { IF I > BuffLen2 }
        Except
          StringPosPChar := Nil;
          BuffPtr2       := Nil;
          i              := BuffLen2 + 1;
          Result         := -1;
        End;  { Try .. Except }
      End;  { While I < BuffLen2  }
   Except
     Result         := -1;
//     StringPosPChar := Nil;
//     BuffPtr2       := Nil;
   End;  { Try .. Except }
End;  { End of Function }


{ ----------------------------------------------------------------------------}
{  ScanFile                                                                   }
{     This function is the guts of File Scan code                             }
{     It performs the actual file scanning.  Note that this code deals with   }
{  files that have null bytes embedded in the file (ie executables, word      }
{  processing documents, etc.)                                                }
{                                                                             }
{ ----------------------------------------------------------------------------}
Function TFileScan.ScanFile(FileName: String):Integer;
Const
   FileError = -1;
var
   FileHandle                   :Integer; { Handle of the file to be scanned }
   ReadCount                    :Integer;
   i                            :Integer;
   FileNamePChar                :PChar;
   SearchStringPChar            :PChar;
   FileBuffer                   :PChar;
   TempFileBuffer               :Pchar;
   SearchPchar                  :PChar;
   BufferSize                   :Word;
   ReadResult                   :Word;
   TempSearchStrings            :TStringList;
   iScanResult                   :Integer;
   TempString                   :String;
   SearchStringsLeft            :Integer;
   SizeofSearchString           :Integer;
   StringsFound                 :Boolean;
   LengthofLongestSearchString  :Integer;
   TempBuffer                   :PChar;
   TempPChar                    :PChar;
   TempBufferSize               :LongInt;
   LengthofSaveString           :Integer;

Begin
     // Initialize Vars to avoid warnings...
     SearchStringPChar     := Nil;
     SizeofSearchString    := 0;

     { Create a copy of the search Strings }
     TempSearchStrings     := TStringList.Create;
     TempSearchStrings.SetText(FSearchStrings.GetText);

    { Initialize appropriate Pointers and setup null terminated strings }
     BufferSize := 30000;   { Read the file in 30,000 byte chunks }
     { Allocate Memory for Null terminated File name }
     GetMem(FileBuffer,BufferSize + 2);  {allocate more to account for the null bytes to be added}
     GetMem(TempFileBuffer,BufferSize + 2);  {allocate more to account for the null bytes to be added}
     { Initialize the FileBuffer to Nulls.... }
//     FillChar(FileBuffer, BufferSize, 0);
//     Initialize(FileBuffer, BufferSize);

     {Now Create the Temporary Buffer and its size - not to be used until the end of first loop }
     TempBufferSize := 10;
     GetMem(TempBuffer,TempBufferSize);


{$IFDEF WIN32}
     FileNamePchar := PChar(FileName);
{$ELSE}
     Getmem(FileNamePChar,Sizeof(FileName));
     FileNamePchar := StrPCopy(FileNamePChar,FileName);
{$ENDIF}

     FileHandle      := _lopen(FileNamePChar,of_Read);
     Try
       { If there is any error opening file - then terminate search }
       If FileHandle = FileError Then Begin
          Result       := -1;
       End Else Begin
       { Scan through the file - terminate search if string found }
         Try
          ReadCount    := 1;
          Result       := 0;
          StringsFound := False;
          { Read the first 30,000 byte chunk of the file }
          ReadResult   := _lread(FileHandle, FileBuffer, BufferSize);
          { Now add the null byte to the end of the chunck that was read from the file }
          SearchPchar  := FileBuffer + ReadResult;
          SearchPChar  := StrPCopy(SearchPChar,Chr(0));
          // Set it back to the start of the file buffer
          SearchPchar  := FileBuffer;

          While ReadResult > 0 do Begin
            For I := 0 to (TempSearchStrings.Count - 1) Do Begin
              Try
                TempString         := TempSearchStrings.Strings[i];
                SizeofSearchString := Length(TempString) + 1;
                GetMem(SearchStringPChar,SizeofSearchString);
                StrPCopy(SearchStringPChar, TempString);

                { Check to see if user has created Event Code }
                If Assigned(FOnBeforeBufferScan) Then FOnBeforeBufferScan(FileName,SearchStringPChar,SearchPChar,ReadCount,FCaseSense);

                { Perform the actual scan of the buffer! }
                iScanResult := ScanBuffer(SearchPChar,SearchStringPChar,StrLen(SearchPChar));

                { Check to see if user has created Event Code }
                If Assigned(FOnAfterBufferScan) Then FOnAfterBufferScan(FileName,SearchStringPChar,SearchPChar,ReadCount,FCaseSense,Result=1);

                IF iScanResult > 0 Then Begin  {If string was found }
                  If ((FSearchType = st_Or) or (FSearchType = st_Not)) Then Begin
                    Result       := 1;
                    StringsFound := True;
                    Break;
                  End Else Begin
                    IF FSearchType = st_And Then Begin
                      TempSearchStrings.strings[i] := 'Flagged for Deletion!';
                    End;
                  End;
                End;

              Finally
                FreeMem(SearchStringPChar,SizeofSearchString);
              End;
            End; { For I := 0 to  ... }

            { Now Delete any found strings from the search list - only for an And search}
            If FSearchType = st_And Then Begin
              i := 0;
              SearchStringsLeft := TempSearchStrings.Count;
              While i < SearchStringsLeft Do Begin
                If TempSearchStrings.strings[i] = 'Flagged for Deletion!' then Begin
                  TempSearchStrings.Delete(i);
                  Dec(SearchStringsLeft);
                End Else Begin
                  Inc(i);
                End;
                If TempSearchStrings.Count = 0 Then Begin
                  StringsFound := True;
                  Result       := 1;
                  Break;
                End;
              End;
            End;

            { If String has been found - then exit read loop }
            If (StringsFound) then begin
               Break;
            End;


            { Now - push the pointer back 'x' bytes in the file - to allow the search
              to cross the old 30,000 byte boundry }

            { Determine the length of the longest search string }
            LengthofLongestSearchString := 0;
            For I := 0 to (TempSearchStrings.Count - 1) do begin
              If Length(TempSearchStrings[i]) > LengthofLongestSearchString then Begin
                 LengthofLongestSearchString := Length(TempSearchStrings[i]);
              End;
            End;

            { This next part is not fun Code!!!!}

            { What I am trying to do here is to take the last chunk from the previsous
              30,000 byte block and add to it the next block read from the file.  This is
              a little tricky - mainly due to the fact that the chunk read from the file
              can contain many null terminated strings - so a simple StrCopy will not
              work.  Ah! - You gotta love playing with Pointers - Can anyone out there say GPF!
            }

            { Determine how big of a chunck to save from the last bufer }
            If ((LengthofLastString - 1) = ReadResult) or (LengthofLastString = ReadResult) then begin
              LengthofSaveString := LengthofLongestSearchString;
            End Else Begin
              LengthofSaveString := LengthofLastString;
            End;
            { Create Temporary Buffer for String }
            FreeMem(TempBuffer,TempBufferSize);
            TempBufferSize := LengthofSaveString + ReadResult + 2;
            GetMem(TempBuffer,TempBufferSize);
            TempPChar := FileBuffer + ReadResult - LengthofSaveString;
            StrCopy(TempBuffer,TempPChar);

            { Read next 30,000 byte chunk from the file }
            ReadResult := _lread(FileHandle, FileBuffer, BufferSize);
            { Now append the next file chunk to the end of the last search }
            StrLCat(TempBuffer,FileBuffer,TempBufferSize);

            { Now add the null byte to the end of the chunck that was read from the file }
            SearchPchar := TempBuffer;
            Inc(ReadCount);  {Used for debugging - I just left it in to tell how many reads..}
          End;  { While ReadResult > 0 }

          { If there was some error (other than end of file) - then return a negative}
          { If the search type is st_Not - and the string is found - this is a failure }
          IF FSearchType = st_Not then begin
            If StringsFound then begin
              Result := 0;
            End Else Begin
              Result := 1;
            End;
          End;
         Except
//           StringPosPChar := Nil;
           Result := -2;
         End; { to Try - Except }
       End;  { IF FileHandle = FileError }

     Finally
     { Dispose of all outstanding variables - and close the file }
       FreeMem(FileBuffer,BufferSize + 2);
       FreeMem(TempBuffer,TempBufferSize);

{$IFNDEF WIN32}
       FreeMem(FileNamePChar,Length(FileName));
{$ENDIF}

       _lclose(FileHandle);
     End; { Try - Finally }
End; { of Function }


{ -------------------------------------------------------------------------}
{  SetCurrInfoLabel                                                        }
{     This function is called to write the FCurrInfoLabel private variable }
{ -------------------------------------------------------------------------}
Procedure TFileScan.SetCurrInfoLabel(NewLabel :TLabel);
begin
   FCurrInfoLabel := NewLabel;
end;


{ -----------------------------------------------------------------------
 This next routine (DirExists) - I have included for your use - I have
 found it useful in many program that deal with the file system - but it
 is not part of this component.
{ -----------------------------------------------------------------------}

{ -----------------------------------------------------------------------}
{  DirExists
     This function determines whether a directory exists or not.
     It is a modified version of DirectoryExists.  Borlands version had
     a bug in it - it did not consider 'C:' to be a directory.  I consider
     any valid drive to be a directory - it is the root directory....
{ -----------------------------------------------------------------------}
{function DirExists(Name: string): Boolean;
var
  SearchRec  :TSearchRec;
  Drive      :Integer;
begin
  if Name[Length(Name)] = '\' then Dec(Name[0]);
  if (Length(Name) = 2) and (Name[2] = ':') then begin
    Name := UpperCase(Name);
    Drive := Ord(Name[1]) - Ord('A');
    If GetDriveType(Drive) > 0 then begin
      Result := True;
    End Else Begin
      Result := False;
    End;
  End else Begin
    Result := FindFirst(Name, faDirectory, SearchRec) = 0;
    If Result then begin
      Result := Result and (SearchRec.Attr and faDirectory <> 0);
    End;
  End;
end;
}
end.
